// ----------------------------------------------------------------------------
/*
 * Copyright (C) 2007 Fabian Greif <fabian.greif@rwth-aachen.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
// ----------------------------------------------------------------------------

#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/crc16.h>

#include <stdio.h>		// DEBUG
#include "defaults.h"

#include "eeprom_functions.h"

/*#if !defined(__AVR_ATmega48__) && !defined(__AVR_ATmega88__) && !defined(__AVR_ATmega168__)
	#error no support for this avr-type!
#endif

// -------------------------------------------------------------------------
static uint8_t EEPROM_read(uint16_t uiAddress)
{
	// Wait for completion of previous write
	while(EECR & (1<<EEPE))
		;
	
	// Set up address register
	EEAR = uiAddress;
	
	// Start eeprom read by writing EERE
	EECR |= (1<<EERE);
	
	// Return data from data register
	return EEDR;
}

// -------------------------------------------------------------------------
static uint16_t EEPROM_read_int(uint16_t uiAddress)
{
	uint16_t temp;
	
	temp = EEPROM_read(uiAddress + 1);			// Lowbyte lesen
	temp |= (EEPROM_read(uiAddress) << 8);		// Highbyte lesen
	
	return temp;
}

// -------------------------------------------------------------------------
static void EEPROM_write(uint16_t uiAddress, uint8_t ucData)
{
	// Wait for completion of previous EEprom write
	while(EECR & (1<<EEPE))
		;
	
	// Set up address and data registers
	EEAR = uiAddress;
	EEDR = ucData;
	
	// SREG Register sichern
	uint8_t temp = SREG;
	cli();
	
	// Write logical one to EEMPE
	EECR |= (1<<EEMPE);
	// Start eeprom write by setting EEPE
	EECR |= (1<<EEPE);
	
	SREG = temp;
}

// -------------------------------------------------------------------------
static void EEPROM_write_int(uint16_t uiAddress, uint16_t uiData)
{
	EEPROM_write( uiAddress, (uint8_t) (uiData >> 8) );			// Highbyte schreiben
	EEPROM_write( uiAddress + 1, (uint8_t) (uiData & 0x00ff) );	// Lowbyte schreiben
}*/

// ----------------------------------------------------------------------------
static uint16_t eeprom_calc_checksum(void)
{
	uint16_t crc = 0xffff;
	for (uint16_t i=3; i < E2END; i++) {
		crc = _crc16_update(crc, eeprom_read_byte((uint8_t *) i));
	}
	
	// dummy read to set eeprom pointer to address 0
	eeprom_read_byte((uint8_t *) 0);
	
	return crc;
}

// ----------------------------------------------------------------------------
void eeprom_store_values(uint8_t *buf, uint8_t size)
{
	// write config-data
	for (uint16_t i=3; i<(size+3); ++i) {
		eeprom_write_byte((uint8_t *) i, *buf++ );
	}
	
	// calculate and save checksum
	eeprom_write_word((uint16_t *) 1, eeprom_calc_checksum() );
	
	// dummy read to set eeprom pointer to address 0
	eeprom_read_byte((uint8_t *) 0);
}

// ----------------------------------------------------------------------------
bool eeprom_load_values(uint8_t *buf, uint8_t size)
{
	/*DEBUG("eeprom-dump\n");
	for (uint16_t i=3; i<(size+3); ++i) {
		DEBUG("%2d: 0x%02x\n", i, eeprom_read_byte((uint8_t *) i));
	}
	
	DEBUG("eeprom-check: %x vs. %x\n", eeprom_calc_checksum(), eeprom_read_word((uint16_t *) 1));*/
	
	if (eeprom_calc_checksum() != eeprom_read_word((uint16_t *) 1))
		return false;
	
	// read config-data
	for (uint16_t i=3; i<(size+3); ++i) {
		*buf++ = eeprom_read_byte((uint8_t *) i);
	}
	
	// dummy read to set eeprom pointer to address 0
	eeprom_read_byte((uint8_t *) 0);
	
	return true;
}
