
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>

#include <util/delay.h>
#include <string.h>

#include "io.h"

tIOValues *io_status;

// ----------------------------------------------------------------------------
struct tIO {
	volatile const uint8_t *pin;
	volatile uint8_t *ddr;
	volatile uint8_t *port;
	uint8_t mask;
} PROGMEM io_def[8] = {
	{ &PINC, &DDRC, &PORTC, (1 << PC5) },
	{ &PINC, &DDRC, &PORTC, (1 << PC4) },
	{ &PINC, &DDRC, &PORTC, (1 << PC3) },
	{ &PINC, &DDRC, &PORTC, (1 << PC2) },
	{ &PIND, &DDRD, &PORTD, (1 << PD2) },
	{ &PINB, &DDRB, &PORTB, (1 << PB5) },
	{ &PINB, &DDRB, &PORTB, (1 << PB4) },
	{ &PINB, &DDRB, &PORTB, (1 << PB3) },
};

// ----------------------------------------------------------------------------
void io_init( tIOValues *val )
{
	io_status = val;
	
	uint8_t i;
	for (i=0; i<8; i++)
	{
		if (val[i].output)
			io_set( 1<<i, val[i].value<<i );
	}
}

// ----------------------------------------------------------------------------
void io_set(uint8_t mask, uint8_t port)
{
	uint8_t i, j;
	
	for (i=0, j=1; i<8; i++, j<<=1)
	{
		if (mask & j)
		{
			// load pin definition
			struct tIO io;
			memcpy_P( &io, &io_def[i], sizeof(struct tIO) );
			
			// set pin as output
			*io.ddr |= io.mask;
			
			cli();
			
			// set clear pin
			if (port & j) {
				*io.port |= io.mask;
			}
			else {
				*io.port &= ~io.mask;
			}
			sei();
			
			// store new config
			io_status[i].output = 1;
			io_status[i].value = (port & j) ? 1 : 0;
		}
	}
}

// ----------------------------------------------------------------------------
uint8_t io_get(uint8_t mask)
{
	uint8_t i, j;
	uint8_t value = 0;
	
	for (i=0, j=1; i<8; i++, j<<=1)
	{
		value <<= 1;
		
		if (mask & j)
		{
			// load pin definition
			struct tIO io;
			memcpy_P( &io, &io_def[i], sizeof(struct tIO) );
			
			// set pin as input
			*io.ddr &= ~io.mask;
			
			// wait a little bit
			_delay_us(1);
			
			// read value
			uint8_t status = (*io.pin & io.mask) ? 1 : 0;
			
			// store new config
			io_status[i].output = 0;
			io_status[i].value = status & 1;
			
			value |= status;
		}
	}
	
	return value;
}

// ----------------------------------------------------------------------------
/*void io_set_timed(uint8_t *buf)
{
	
	
	
}*/
