// ----------------------------------------------------------------------------
/*
 * Copyright (C) 2007 Fabian Greif <fabian.greif@rwth-aachen.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
// ----------------------------------------------------------------------------
 
#include <avr/pgmspace.h>

#include <stdio.h>		// for debugging
#include "defaults.h"

#include "scanner.h"

enum {
	IDLE,
	READ_COMMAND,
	READ_DATA
} state;

static uint8_t buffer[MAX_COMMAND_LENGTH];
static uint8_t buffer_pos;

static const tCommandDefinition *command_table;
static uint8_t command_table_size;

static uint8_t command_length;
static tFuncPtr command_function;


// ----------------------------------------------------------------------------
void scanner_init(const tCommandDefinition *table, uint8_t table_size)
{
	command_table = table;
	command_table_size = table_size;
	
	state = IDLE;
	
	DEBUG("%d commands\n", command_table_size);
}

// ----------------------------------------------------------------------------
void scanner_scan(uint8_t chr)
{
	tCommandDefinition def;
	uint8_t i;
	
	switch (state)
	{
		case IDLE:
			if (chr == '#')
				state = READ_COMMAND;
			break;
		
		case READ_COMMAND:
			if (chr == '#')
				break;
			
			DEBUG("search %c (0x%x)\n", chr, chr);
			
			state = IDLE;
			for (i = 0; i < command_table_size; i++)
			{
				memcpy_P( &def, &command_table[i], sizeof(tCommandDefinition));
				
				if (def.chr == chr)
				{
					DEBUG("command %d\n", i);
					
					if (def.length == 0)
					{
						// commands which don't need additional data
						// can be executed immediately
						def.ptr( (uint8_t *) 0 );
						state = IDLE;
						
						PINC |= (1<<PC0);
						DEBUG("Ok\n");
					}
					else
					{
						command_length = def.length;
						command_function = def.ptr;
						buffer_pos = 0;
						
						state = READ_DATA;
					}
					break;
				}
			}
			break;
			
		case READ_DATA:
			buffer[buffer_pos] = chr;
			buffer_pos++;
			
			if (buffer_pos == command_length)
			{
				// execute command
				command_function( buffer );
				state = IDLE;
				
				PINC |= (1<<PC0);
				DEBUG("Ok\n");
			}
			break;
		
		default:
			state = IDLE;
			break;
	}
}
