#ifndef	SCANNER_H
#define	SCANNER_H

// ----------------------------------------------------------------------------
/*
 * Copyright (C) 2007 Fabian Greif <fabian.greif@rwth-aachen.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * \file	scanner.h
 * \brief	Scanner for simple commands
 *
 *
 * \date	3. Sep 2007
 * \author	Fabian Greif <fabian.greif@rwth-aachen.de>
 */
// ----------------------------------------------------------------------------

#include <inttypes.h>

#define	MAX_COMMAND_LENGTH	10		//!< max. length of the data field for the commands

// ----------------------------------------------------------------------------
typedef void (*tFuncPtr)(uint8_t *buf);

// ----------------------------------------------------------------------------
typedef struct {
	char chr;
	uint8_t length;
	tFuncPtr ptr;
} tCommandDefinition;

// ----------------------------------------------------------------------------
/**
 * \brief	initialize scanner
 *
 * \param	*table		Pointer to the command table
 * \param	table_size	Size of the command table
 * \return	None
 */
void scanner_init(const tCommandDefinition *table, uint8_t table_size);

// ----------------------------------------------------------------------------
/**
 * \brief	scan a character
 *
 * \param	chr		Character which will be checkes
 */
void scanner_scan(uint8_t chr);


#endif	// SCANNER_H
