#ifndef	SERVO_H
#define	SERVO_H

// ----------------------------------------------------------------------------
/*
 * Copyright (C) 2007 Fabian Greif <fabian.greif@rwth-aachen.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
// ----------------------------------------------------------------------------

#include <inttypes.h>

// ----------------------------------------------------------------------------
#if (F_CPU == 4000000)
	// Zeiten => Timerwerte bei 4 MHz
	#define SIG_0_5MS	0xf830
	#define	SIG_0_8MS	0xf380
	#define	SIG_1_0MS	0xf060
	#define	SIG_1_5MS	0xe890
	#define	SIG_2_0MS	0xe0c0
	#define	SIG_2_2MS	0xdda0
	#define	SIG_2_5MS	0xd8f0
	
	#define	TIMER0_RELOAD	0xd9
#elif (F_CPU == 7372800)
	// Zeiten => Timerwerte bei 7,3728 MHz
	#define SIG_0_5MS	0xf19a
	#define	SIG_0_8MS	0xe8f6
	#define	SIG_1_0MS	0xe334
	#define	SIG_1_5MS	0xd4cd
	#define	SIG_2_0MS	0xc667
	#define	SIG_2_2MS	0xc0a4
	#define	SIG_2_5MS	0xb800
	
	#define	TIMER0_RELOAD	0xb8
#elif (F_CPU == 8000000)
	// Zeiten => Timerwerte bei 8 MHz
	#define SIG_0_5MS	0xf060
	#define	SIG_0_8MS	0xe700
	#define	SIG_1_0MS	0xe0c0
	#define	SIG_1_5MS	0xd120
	#define	SIG_2_0MS	0xc180
	#define	SIG_2_2MS	0xbb40
	#define	SIG_2_5MS	0xb1e0
	
	#define	TIMER0_RELOAD	0xb2
#elif (F_CPU == 14745600)
	// Zeiten => Timerwerte bei 14,7456 MHz
	#define SIG_0_5MS	0xe335
	#define	SIG_0_8MS	0xd1ed
	#define	SIG_1_0MS	0xc669
	#define	SIG_1_5MS	0xa99b
	#define	SIG_2_0MS	0x8ccf
	#define	SIG_2_2MS	0x8149
	#define	SIG_2_5MS	0x7001
	
	#define	TIMER0_RELOAD	0x70
#else
	#error	Die gewaehlte Taktfrequenz wird nicht unterstuetzt.
#endif

#define	NUM_SERVOS	8

// ----------------------------------------------------------------------------
typedef struct {
	uint16_t max;
	uint16_t min;
	int8_t offset;
	int8_t position;
} tServoValues;

// ----------------------------------------------------------------------------
extern void servo_init(tServoValues def[]);

// ----------------------------------------------------------------------------
extern void servo_set_position(uint8_t servo, int8_t pos);

// ----------------------------------------------------------------------------
extern int8_t servo_get_position(uint8_t servo);

// ----------------------------------------------------------------------------
extern void servo_set_offset(uint8_t servo, int8_t offset);

// ----------------------------------------------------------------------------
extern void servo_set_limits(uint8_t servo, uint16_t max, uint16_t min);

#endif	// SERVO_H
