#ifndef	WRAPPER_H
#define	WRAPPER_H

// ----------------------------------------------------------------------------
void servo_set_position_wrapper(uint8_t *buf) {
	servo_set_position( buf[0], (int8_t) buf[1] - 128  );
}

void servo_get_position_wrapper(uint8_t *buf) {
	uart_putc( servo_get_position( buf[0] ) + 128 );
}

void servo_set_offset_wrapper(uint8_t *buf) {
	servo_set_offset( buf[0], buf[1] );
}

void servo_set_limits_wrapper(uint8_t *buf) {
	uint16_t max, min;
	max = (buf[1] << 8) | buf[2];
	min = (buf[3] << 8) | buf[4];
	servo_set_limits( buf[0], max, min );
}

// ----------------------------------------------------------------------------
void set_pins_wrapper(uint8_t *buf) {
	io_set( buf[0], buf[1] );
}

void get_pins_wrapper(uint8_t *buf) {
	uart_putc( io_get( buf[0] ) );
}
//void set_timed_pins_wrapper(uint8_t *buf) {}

// ----------------------------------------------------------------------------
void get_version(uint8_t *buf) {
	uart_putc(VERSION_HIGH);
	uart_putc(VERSION_LOW);
}

// ----------------------------------------------------------------------------
void set_baudrate(uint8_t *buf)
{
	uint16_t value = 0xffff;
	
	#if (F_CPU == 4000000)
	switch (buf[0]) {
		case 0: value = 103; break;		// 2400
		case 1: value = 51;  break;		// 4800
		case 2: value = 25;  break;		// 9600
		case 4: value = 12;  break;		// 19.2k
		default: break;
	}
	#elif (F_CPU == 7372800)
	switch (buf[0]) {
		case 0: value = 191; break;		// 2400
		case 1: value = 95;  break;		// 4800
		case 2: value = 47;  break;		// 9600
		case 3: value = 31;  break;		// 14.4k
		case 4: value = 23;  break;		// 19.2k
		case 5: value = 15;  break;		// 28.8k
		case 6: value = 11;  break;		// 38.4k
		case 7: value = 7;   break;		// 57.6k
		case 8: value = 3;   break;		// 115.2k
		default: break;
	}
	#elif (F_CPU == 8000000)
	switch (buf[0]) {
		case 0: value = 207; break;		// 2400
		case 1: value = 103; break;		// 4800
		case 2: value = 51;  break;		// 9600
		case 3: value = 34;  break;		// 14.4k (0,8% error)
		case 4: value = 25;  break;		// 19.2k
		case 6: value = 12;  break;		// 38.4k
		default: break;
	}
	#elif (F_CPU == 14745600)
	switch (buf[0]) {
		case 0: value = 383; break;		// 2400
		case 1: value = 191;  break;	// 4800
		case 2: value = 95;  break;		// 9600
		case 3: value = 63;  break;		// 14.4k
		case 4: value = 47;  break;		// 19.2k
		case 5: value = 31;  break;		// 28.8k
		case 6: value = 23;  break;		// 38.4k
		case 7: value = 15;   break;	// 57.6k
		case 8: value = 7;   break;		// 115.2k
		default: break;
	}
	#endif
	
	if (value != 0xffff) {
		command_values.baud = value;
		uart_init( value );
	}
}

// ----------------------------------------------------------------------------
void save_values_to_eeprom_wrapper(uint8_t *buf) {
	eeprom_store_values( (uint8_t *) &command_values, sizeof(tCommandValues) );
}

void load_default_values(void) {
	memcpy_P( &command_values, &command_values_defaults, sizeof(tCommandValues) );
}

void load_default_values_wrapper(uint8_t *buf) {
	load_default_values();
}

// ----------------------------------------------------------------------------
void reset_wrapper(uint8_t *buf)
{
	cli();
	wdt_enable(WDTO_15MS);
	
	for(;;);
}

#endif	// WRAPPER_H
